"""This module downloads fibber datasets.

To download preprocessed datasets (Recommended), run::

    python -m fibber.datasets.download_datasets

To download datasets from their original sources, and preprocess them locally::

    python -m fibber.datasets.download_datasets --process_raw 1
"""


import argparse
import glob
import json
import os

from fibber import get_root_dir, log
from fibber.datasets import (
    preprocess_ag, preprocess_imdb, preprocess_mnli, preprocess_mr, preprocess_snli,
    preprocess_yelp)
from fibber.datasets.dataset_utils import verify_dataset
from fibber.datasets.downloadable_datasets import downloadable_dataset_urls
from fibber.download_utils import download_file

logger = log.setup_custom_logger(__name__)

parser = argparse.ArgumentParser()

parser.add_argument("--process_raw", choices=["0", "1"], default="0",
                    help="Use 1 to download and process raw data on this machine. "
                    "Use 0 (Default) to download processed data.")
parser.add_argument("--verify", choices=["0", "1"], default="1",
                    help="Verify each json in each datasets have proper attributes.")

DATASET_PREPROCESS_FN = {
    "ag": preprocess_ag.download_and_preprocess_ag,
    "imdb": preprocess_imdb.download_and_preprocess_imdb,
    "mnli": preprocess_mnli.download_and_preprocess_mnli,
    "mr": preprocess_mr.download_and_preprocess_mr,
    "snli": preprocess_snli.download_and_preprocess_snli,
    "yelp": preprocess_yelp.download_and_preprocess_yelp
}

if __name__ == "__main__":
    FLAGS = parser.parse_args()

    if FLAGS.process_raw == "1":
        for name, processing_func in DATASET_PREPROCESS_FN.items():
            logger.info("Start download and process %s.", name)
            processing_func()

    else:
        download_file(subdir="", **downloadable_dataset_urls["processed-datasets"])

    if FLAGS.verify == "1":
        root_dir = get_root_dir()
        datasets_dir = os.path.join(root_dir, "datasets")
        dataset_json_list = sorted(glob.glob(datasets_dir + "/*/*.json"))
        for json_filename in dataset_json_list:
            logger.info("Verify %s.", json_filename)
            with open(json_filename) as f:
                data = json.load(f)

            verify_dataset(data)



import glob
import json

import tqdm

from fibber import log
from fibber.datasets.preprocess_utils import download_raw_and_preprocess

logger = log.setup_custom_logger(__name__)


def preprocess_imdb_data(input_folder, output_filename):
    """preprocess raw IMDB dataset folder to Fibber's JSON format."""

    logger.info("Start preprocessing data, and save at %s.", output_filename)
    data = {
        "label_mapping": ["negative", "positive"],
        "cased": True,
        "paraphrase_field": "text0",
    }

    datalist = []

    for filename in tqdm.tqdm(glob.glob(input_folder + "/neg/*.txt")):
        with open(filename, encoding="utf-8", errors="ignore") as f:
            text = "".join(f.readlines()).strip().replace("<br />", "")
            datalist.append({
                "label": 0,
                "text0": text
            })

    for filename in tqdm.tqdm(glob.glob(input_folder + "/pos/*.txt")):
        with open(filename, encoding="utf-8", errors="ignore") as f:
            text = "".join(f.readlines()).strip().replace("<br />", "")
            datalist.append({
                "label": 1,
                "text0": text
            })

    data["data"] = datalist
    with open(output_filename, "w") as f:
        json.dump(data, f, indent=2)


def download_and_preprocess_imdb():
    """Download and preprocess IMDB sentiment classification dataset. """
    download_raw_and_preprocess(
        dataset_name="imdb",
        download_list=["imdb-raw"],
        preprocess_fn=preprocess_imdb_data,
        preprocess_input_output_list=[
            ("raw/aclImdb/train", "train.json"),
            ("raw/aclImdb/test", "test.json")])


if __name__ == "__main__":
    download_and_preprocess_imdb()



"""This metric outputs the word-level editing distance between original text and paraphrased
text.
"""

import re

import numpy as np

from fibber.metrics.metric_base import MetricBase


class EditDistanceMetric(MetricBase):
    """This class measures the editing distance between two sentences."""

    def __init__(self, editing_distance_ignore_punctuation=True, **kargs):
        """Initialize.

        Args:
            editing_distance_ignore_punctuation (bool): whether to ignore punctuation when
                computing editing distance.
        """
        super(EditDistanceMetric, self).__init__()
        self._no_puctuation = editing_distance_ignore_punctuation

    def measure_example(self, origin, paraphrase, data_record=None, paraphrase_field="text0"):
        """compute editing distance between original and parapharse.

        Args:
            origin (str): original text.
            paraphrase (str): paraphrased text.
            data_record: ignored.
            paraphrase_field: ignored.

        Returns:
            (int): the editing distance.
        """
        if self._no_puctuation:
            origin = re.sub(r"[^a-zA-Z0-9]", " ", origin)
            paraphrase = re.sub(r"[^a-zA-Z0-9]", " ", paraphrase)

        origin = origin.split()
        paraphrase = paraphrase.split()
        if len(origin) == 0 or len(paraphrase) == 0:
            return len(origin) + len(paraphrase)

        f = np.zeros((len(origin) + 1, len(paraphrase) + 1), dtype='int')
        for i in range(len(origin)):
            for j in range(len(paraphrase)):
                f[i + 1][j + 1] = min(f[i, j + 1] + 1, f[i + 1, j] + 1, f[i, j] + 1)
                if origin[i] == paraphrase[j]:
                    f[i + 1][j + 1] = min(f[i + 1][j + 1], f[i][j])

        return int(f[len(origin)][len(paraphrase)])



import os

from fibber.datasets.downloadable_datasets import downloadable_dataset_urls
from fibber.download_utils import download_file, get_root_dir


def download_raw_and_preprocess(dataset_name,
                                download_list,
                                preprocess_fn,
                                preprocess_input_output_list):
    """Download and preprocess raw data into fibber's format.

    Args:
        dataset_name (str): the name of the dataset.
        download_list ([str]): a list of strings indicating which file to download. Each
            element in this list should corresponds to a one key in ``downloadable_dataset_urls``.
        preprocess_fn (fn): a function to preprocess the dataset.
        preprocess_input_output_list ([(str, str), ...]): A list of tuples. Each tuple indicate a
            pair of input and output file or path name.
    """
    root_dir = get_root_dir()
    dataset_dir = "datasets/" + dataset_name

    for item in download_list:
        download_file(**downloadable_dataset_urls[item], subdir=os.path.join(dataset_dir, "raw"))

    for input_name, output_name in preprocess_input_output_list:
        preprocess_fn(os.path.join(root_dir, dataset_dir, input_name),
                      os.path.join(root_dir, dataset_dir, output_name))



downloadable_resource_urls = {
    "default-glove-embeddings": {
        "filename": "glove.tgz",
        "url": "https://fibber-data.s3.amazonaws.com/glove.tgz",
        "md5": "7deac3ce2cfd8be24f97bc4ba007a19c",
        "untar": True,
        "unzip": False,
    },
    "default-stopwords": {
        "filename": "stopwords.txt",
        "url": "https://fibber-data.s3.amazonaws.com/stopwords_v1.txt",
        "md5": "6d3c778d216390063f424112087e5c10",
        "untar": False,
        "unzip": False,
    },
    "nltk-punkt": {
        "filename": "punkt.zip",
        "url": "https://fibber-data.s3.amazonaws.com/punkt.zip",
        "md5": "398bbed6dd3ebb0752fe0735d1c418fe",
        "untar": False,
        "unzip": True,
    },
    "nltk_stopwords": {
        "filename": "stopwords.zip",
        "url": "https://fibber-data.s3.amazonaws.com/nltk_stopwords.zip",
        "md5": "884694b9055d1caee8a0ca3aa3b2c7f7",
        "untar": False,
        "unzip": True,
    },
    "universal-sentence-encoder": {
        "filename": "universal-sentence-encoder-large_5.tar.gz",
        "url": "https://fibber-data.s3.amazonaws.com/universal-sentence-encoder-large_5.tar.gz",
        "md5": "1ffeec98147fd21cc2d5627ec09f0d57",
        "untar": True,
        "unzip": False,
    },
    "bert-base-cased": {
        "filename": "bert-base-cased.tgz",
        "url": "https://fibber-data.s3.amazonaws.com/bert-base-cased.tgz",
        "md5": "24e9ed53ff28518793de40c344ba0b6a",
        "untar": True,
        "unzip": False,
    },
    "bert-base-uncased": {
        "filename": "bert-base-uncased.tgz",
        "url": "https://fibber-data.s3.amazonaws.com/bert-base-uncased.tgz",
        "md5": "0c41b10e7dd3a42cfd314e2bab315b25",
        "untar": True,
        "unzip": False,
    },
    "gpt2-medium": {
        "filename": "gpt2-medium.tgz",
        "url": "https://fibber-data.s3.amazonaws.com/gpt2-medium.tgz",
        "md5": "2de5082b58bc7a2d2fe4f744db2fdafc",
        "untar": True,
        "unzip": False,
    },
    "stanford-corenlp": {
        "filename": "stanford-corenlp-4.1.0.zip",
        "url": "https://fibber-data.s3.amazonaws.com/stanford-corenlp-4.1.0.zip",
        "md5": "2ee2e614eec7f913800957297139f63d",
        "untar": False,
        "unzip": True,
    },
    "bert-base-uncased-clf-demo": {
        "filename": "bert-base-uncased-clf-demo.tgz",
        "url": "https://fibber-data.s3.amazonaws.com/bert-base-uncased-clf-demo.tgz",
        "md5": "4af6ca998625bbd64d9a6d59a3af035f",
        "untar": True,
        "unzip": False,
    },
    "bert-base-uncased-lm-demo": {
        "filename": "bert-base-uncased-lm-demo.tgz",
        "url": "https://fibber-data.s3.amazonaws.com/bert-base-uncased-lm-demo.tgz",
        "md5": "ff8f302c81b0aace15890842b7bb0e17",
        "untar": True,
        "unzip": False
    },
    "wpe-demo": {
        "filename": "wpe-demo.tgz",
        "url": "https://fibber-data.s3.amazonaws.com/wpe-demo.tgz",
        "md5": "673fb249c2d63e334bc224c4a7f9bc89",
        "untar": True,
        "unzip": False
    }
}



import pytest
import torch

from fibber.benchmark import Benchmark
from fibber.paraphrase_strategies import BertSamplingStrategy, TextAttackStrategy


@pytest.fixture()
def gpu_id():
    if torch.cuda.device_count() > 0:
        return 0
    return -1


@pytest.mark.slow
def test_integrity_identity(gpu_id):
    torch.cuda.empty_cache()
    benchmark = Benchmark(
        output_dir="exp-pytest",
        dataset_name="mr",
        subsample_attack_set=100,
        use_gpu_id=gpu_id,
        gpt2_gpu_id=gpu_id,
        bert_gpu_id=gpu_id,
        bert_clf_steps=1000,
        bert_clf_bs=32
    )

    result = benchmark.run_benchmark(paraphrase_strategy="IdentityStrategy")
    assert result["3_BertClassifier_AfterAttackAccuracy_USE_0.85_GPT_5.0(↓)"] > 0.85


@pytest.mark.slow
def test_integrity_textfooler(gpu_id):
    torch.cuda.empty_cache()
    benchmark = Benchmark(
        output_dir="exp-pytest",
        dataset_name="mr",
        subsample_attack_set=100,
        use_gpu_id=gpu_id,
        gpt2_gpu_id=gpu_id,
        bert_gpu_id=gpu_id,
        bert_clf_steps=1000,
        bert_clf_bs=32
    )

    strategy = TextAttackStrategy(
        arg_dict={"ta_recipe": "TextFoolerJin2019"},
        dataset_name="mr",
        strategy_gpu_id=gpu_id,
        output_dir="exp-pytest",
        metric_bundle=benchmark.get_metric_bundle())
    result = benchmark.run_benchmark(paraphrase_strategy=strategy)

    assert result["3_BertClassifier_AfterAttackAccuracy_USE_0.85_GPT_5.0(↓)"] < 0.50


@pytest.mark.slow
def test_integrity_bertsampling(gpu_id):
    torch.cuda.empty_cache()
    benchmark = Benchmark(
        output_dir="exp-pytest",
        dataset_name="mr",
        subsample_attack_set=1,
        use_gpu_id=gpu_id,
        gpt2_gpu_id=gpu_id,
        bert_gpu_id=gpu_id,
        bert_clf_steps=1000,
        bert_clf_bs=32
    )
    strategy = BertSamplingStrategy(
        arg_dict={"bs_lm_steps": 1000},
        dataset_name="mr",
        strategy_gpu_id=gpu_id,
        output_dir="exp-pytest",
        metric_bundle=benchmark.get_metric_bundle())
    benchmark.run_benchmark(paraphrase_strategy=strategy, num_paraphrases_per_text=10)



import copy
import datetime
import json

import torch
import tqdm

from fibber import log

logger = log.setup_custom_logger(__name__)


class StrategyBase(object):
    """The base class for all paraphrase strategies.

    The simplest way to write a strategy is to overwrite the ``paraphrase_example`` function. This
    function takes one data records, and returns multiple paraphrases of a given field.

    For more advanced use cases, you can overwrite the ``paraphrase`` function.

    Some strategy may have hyper-parameters. Add hyper parameters into the class attribute
    ``__hyperparameters__``.

    Hyperparameters defined in ``__hyperparameters__`` can be added to the command line arg parser
    by ``add_parser_args(parser)``. The value of the hyperparameters will be added to
    ``self._strategy_config``.

    Attributes:
        __abbr__ (str): a unique string as an abbreviation for the strategy.
        __hyper_parameters__ (list): A list of tuples that defines the hyperparameters for the
            strategy. Each tuple is ``(name, type, default, help)``. For example::

                __hyperparameters = [ ("p1", int, -1, "the first hyper parameter"), ...]
    """
    __abbr__ = "base"
    __hyperparameters__ = []

    def __init__(self, arg_dict, dataset_name, strategy_gpu_id, output_dir, metric_bundle):
        """Initialize the paraphrase_strategies.

        This function initialize the ``self._strategy_config``, ``self._metric_bundle``,
        ``self._device``, ``self._output_dir``, ``self._dataset_name``.

        **You should not overwrite this function.**

        * self._strategy_config (dict): a dictionary that stores the strategy name and all
          hyperparameter values. The dict is also saved to the results.
        * self._metric_bundle (MetricBundle): the metrics that will be used to evaluate
          paraphrases. Strategies can compute metrics during paraphrasing.
        * self._device (torch.Device): any computation that requires a GPU accelerator should
          use this device.
        * self._output_dir (str): the dir name where the strategy can save files.
        * self._dataset_name (str): the dataset name.

        Args:
            arg_dict (dict): all args load from command line.
            dataset_name (str): the name of the dataset.
            strategy_gpu_id (int): the gpu id to run the strategy.
            output_dir (str): a directory to save any models or temporary files.
            metric_bundle (MetricBundle): a MetricBundle object.
        """
        super(StrategyBase, self).__init__()

        # paraphrase_strategies config will be saved to the results.
        self._strategy_config = dict()

        for p_name, p_type, p_default, p_help in self.__hyperparameters__:
            arg_name = "%s_%s" % (self.__abbr__, p_name)
            if arg_name not in arg_dict:
                logger.warning("%s_%s not found in args.", self.__abbr__, p_name)
                p_value = p_default
            else:
                p_value = arg_dict[arg_name]

            assert p_name not in self._strategy_config
            self._strategy_config[p_name] = p_value

        self._strategy_config["strategy_name"] = str(self)

        self._metric_bundle = metric_bundle

        if strategy_gpu_id == -1:
            logger.warning("%s is running on CPU." % str(self))
            self._device = torch.device("cpu")
        else:
            logger.info("%s is running on GPU %d.", str(self), strategy_gpu_id)
            self._device = torch.device("cuda:%d" % strategy_gpu_id)

        self._output_dir = output_dir
        self._dataset_name = dataset_name

    def __repr__(self):
        return self.__class__.__name__

    @classmethod
    def add_parser_args(cls, parser):
        """create commandline args for all hyperparameters in ``__hyperparameters__``.

        Args:
            parser: an arg parser.
        """
        logger.info("%s has %d args to set from command line.",
                    cls.__name__, len(cls.__hyperparameters__))

        for p_name, p_type, p_default, p_help in cls.__hyperparameters__:
            parser.add_argument("--%s_%s" % (cls.__abbr__, p_name), type=p_type,
                                default=p_default, help=p_help)

    def fit(self, trainset):
        """Fit the paraphrase strategy on a training set.

        Args:
            trainset (dict): a fibber dataset.
        """
        logger.info("Training is needed for this strategy. Did nothing.")

    def paraphrase_example(self, data_record, field_name, n):
        """Paraphrase one data record.

        This function should be overwritten by subclasses. When overwriting this class, you can
        use ``self._strategy_config``, ``self._metric_bundle``,  ``self._device``,
        ``self._output_dir``, and ``self._dataset_name``

        Args:
            data_record (dict): a dict storing one data of a dataset.
            field_name (str): the field needed to be paraphrased.
            n (int): number of paraphrases.

        Returns:
            ([str,]): A list contain at most n strings.
        """
        raise NotImplementedError()

    def paraphrase_dataset(self, paraphrase_set, n, tmp_output_filename):
        """Paraphrase one dataset.

        Args:
            paraphrase_set (dict): a dict storing one data of a dataset.
            n (int): number of paraphrases.
            tmp_output_filename (str): the output json filename to save results during running.

        Returns:
            (dict): A dict containing the original text and paraphrased text.
        """
        results = copy.deepcopy(dict([(k, v) for k, v in paraphrase_set.items() if k != "data"]))
        results["strategy_config"] = self._strategy_config
        results["data"] = []

        last_tmp_output_save_time = -1

        for data_record in tqdm.tqdm(paraphrase_set["data"]):
            data_record = copy.deepcopy(data_record)
            data_record[paraphrase_set["paraphrase_field"] + "_paraphrases"] = (
                self.paraphrase_example(
                    data_record, paraphrase_set["paraphrase_field"], n)[:n])
            results["data"].append(data_record)

            # save tmp output every 30 seconds
            if datetime.datetime.now().timestamp() - last_tmp_output_save_time > 30:
                with open(tmp_output_filename, "w") as f:
                    json.dump(results, f, indent=2)
                last_tmp_output_save_time = datetime.datetime.now().timestamp()

        with open(tmp_output_filename, "w") as f:
            json.dump(results, f, indent=2)

        return results



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md', encoding='utf-8') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md', encoding='utf-8') as history_file:
    history = history_file.read()

install_requires = [
    'numpy>=1.18.0',
    'tensorflow-gpu>=2.0.0',
    'tensorflow-hub>=0.9.0',
    'torch<2,>=1.0',
    'torchvision<1,>=0.4.2',
    'transformers>=2.4.0',
    'tqdm>=4.0.0',
    'spacy>=2.0.0',
    'pandas>=1.0.0',
    'nltk>=3.0',
    'stanza>=1.1.0',
    'sentence-transformers>=0.3.0'
]

setup_requires = [
    'pytest-runner>=2.11.1',
]

tests_require = [
    'pytest>=3.4.2',
    'pytest-cov>=2.6.0',
]

development_requires = [
    # general
    'bumpversion>=0.5.3',
    'pip>=9.0.1',
    'watchdog>=0.8.3',

    # docs
    'm2r2>=0.2.5,<0.3',
    'nbsphinx>=0.5.0,<0.7',
    'Sphinx==3.2.1',
    'pydata-sphinx-theme',
    'autodocsumm>=0.1.10',
    'PyYaml>=5.3.1,<6',
    'argh>=0.26.2,<1',
    'sphinx_rtd_theme>=0.4,<1',
    'ipython>=7,<8',

    # style check
    'flake8>=3.7.7',
    'isort>=5',

    # fix style issues
    'autoflake>=1.2',
    'autopep8>=1.4.3',

    # distribute on PyPI
    'twine>=1.10.0',
    'wheel>=0.30.0',

    # Advanced testing
    'coverage>=4.5.1',
    'tox>=2.9.1',
]

setup(
    author='MIT Data To AI Lab',
    author_email='dailabmit@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description='Fibber is a benchmarking suite for adversarial attacks on text classification.',
    extras_require={
        'test': tests_require,
        'dev': development_requires + tests_require,
    },
    install_package_data=True,
    install_requires=install_requires,
    license='MIT license',
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='fibber fibber fibber',
    name='fibber',
    packages=find_packages(include=['fibber', 'fibber.*']),
    python_requires='>=3.6',
    setup_requires=setup_requires,
    test_suite='tests',
    tests_require=tests_require,
    url='https://github.com/DAI-Lab/fibber',
    version='0.2.2',
    zip_safe=False,
)



import pytest
import torch

from fibber.metrics.edit_distance_metric import EditDistanceMetric
from fibber.metrics.glove_semantic_similarity_metric import GloVeSemanticSimilarityMetric
from fibber.metrics.gpt2_grammar_quality_metric import GPT2GrammarQualityMetric
from fibber.metrics.sbert_semantic_similarity_metric import SBERTSemanticSimilarityMetric
from fibber.metrics.use_semantic_similarity_metric import USESemanticSimilarityMetric


@pytest.fixture
def gpu_id():
    if torch.cuda.device_count() > 0:
        return 0
    return -1


def metric_test_helper(metric, io_pairs, batched_io_pairs, eps=0):
    for (origin, paraphrase), true_output in io_pairs:
        value = metric.measure_example(origin, paraphrase)
        assert isinstance(value, int) or isinstance(value, float)
        assert abs(value - true_output) <= eps

    for (origin, paraphrase_list), true_output_list in batched_io_pairs:
        values = metric.measure_batch(origin, paraphrase_list)
        assert isinstance(values, list)
        assert all([isinstance(x, int) or isinstance(x, float) for x in values])
        assert all([abs(output - true_output) <= eps
                    for output, true_output in
                    zip(values, true_output_list)])


def test_edit_distance_metric():
    io_pairs = [
        (("aa bb xx dd zz xx ee", "aa bb xy yz dd xy ee"), 4),
        (("aa bb cc dd", "aa bb dd"), 1),
        (("aa bb cc", " aa  bb  cc "), 0),
        (("a. b; c?", "a1, b& c'"), 1)
    ]
    batched_io_pairs = [
        (("aa bb cc", ["aa bb cc dd", "aa bb cc", "aa ee"]), [1, 0, 2])
    ]
    editing_distance_metric = EditDistanceMetric()

    metric_test_helper(editing_distance_metric, io_pairs, batched_io_pairs)


@pytest.mark.slow
def test_use_semantic_similarity(gpu_id):
    io_pairs = [
        (("Sunday is the first day in a week.",
          "Obama was the president of the United State."), 0.171),
        (("Sunday is the first day in a week",
          "Saturday is the last day in a week"), 0.759)
    ]
    batched_io_pairs = [
        (("Sunday is the first day in a week",
          ["Obama was the president of the United State.",
           "Saturday is the last day in a week"]),
         [0.171, 0.759]),
    ]
    use_semantic_similarity_metric = USESemanticSimilarityMetric(use_gpu_id=gpu_id)
    metric_test_helper(use_semantic_similarity_metric, io_pairs, batched_io_pairs, eps=0.01)


@pytest.mark.slow
def test_sbert_semantic_similarity(gpu_id):
    io_pairs = [
        (("Sunday is the first day in a week.",
          "Obama was the president of the United State."), 0.326),
        (("Sunday is the first day in a week",
          "Saturday is the last day in a week"), 0.619)
    ]
    batched_io_pairs = [
        (("Sunday is the first day in a week",
          ["Obama was the president of the United State.",
           "Saturday is the last day in a week"]),
         [0.326, 0.619]),
    ]
    sbert_semantic_similarity_metric = SBERTSemanticSimilarityMetric(sbert_gpu_id=gpu_id)
    metric_test_helper(sbert_semantic_similarity_metric, io_pairs, batched_io_pairs, eps=0.01)


@pytest.mark.slow
def test_gpt2_grammar_quality(gpu_id):
    io_pairs = [
        (("Sunday is the first day in a week.",
          "Sunday is is is first daay in a week."), 14.64),
        (("Sunday is the first day in a week.",
          "Saturday is the last day in a week."), 1.10)
    ]
    batched_io_pairs = [
        (("Sunday is the first day in a week.",
          ["Sunday is is is first daay in a week.",
           "Saturday is the last day in a week."]),
         [14.64, 1.10]),
    ]
    gpt2_grammar_quality_metric = GPT2GrammarQualityMetric(gpt2_gpu_id=gpu_id)
    metric_test_helper(gpt2_grammar_quality_metric, io_pairs, batched_io_pairs, eps=0.1)


@pytest.mark.slow
def test_glove_semantic_similarity():
    io_pairs = [
        (("the a the to", "he him"), 0.0),
        (("Saturday is the last day in a week.",
          "Sunday is the last day in a week."), 0.997),
        (("Saturday is the last day in a week.",
          "Obama was the president of the United State."), 0.678),
    ]
    batched_io_pairs = [
        (("Saturday is the last day in a week.",
          ["Sunday is the last day in a week.",
           "Obama was the president of the United State."]),
         [0.997, 0.678])
    ]
    glove_semantic_similarity_metric = GloVeSemanticSimilarityMetric()
    metric_test_helper(glove_semantic_similarity_metric, io_pairs, batched_io_pairs, eps=0.01)



import os

import pandas as pd

from fibber import get_root_dir


def update_detailed_result(aggregated_result, result_dir=None):
    """Read dataset detailed results and add a row to the file. Create a new file if the table
    does not exist.

    Args:
        aggregated_result (dict): the aggregated result as a dict.
        result_dir (str or None): the directory to save results. If None, use
            ``<fibber_root_dir>/results/``.
    """
    if result_dir is None:
        result_dir = os.path.join(get_root_dir(), "results")
    os.makedirs(result_dir, exist_ok=True)
    result_filename = os.path.join(result_dir, "detail.csv")
    if os.path.exists(result_filename):
        results = pd.read_csv(result_filename)
    else:
        results = pd.DataFrame()

    results = results.append(aggregated_result, ignore_index=True)
    results.to_csv(result_filename, index=False)


def load_detailed_result():
    """Read detailed results from file.

    Returns:
        (pandas.DataFrame): the detailed result table. Returns an empty DataFrame if file does not
        exist.
    """
    result_dir = os.path.join(get_root_dir(), "results")
    result_filename = os.path.join(result_dir, "detail.csv")
    if os.path.exists(result_filename):
        return pd.read_csv(result_filename)
    else:
        return pd.DataFrame()


def update_overview_result(overview_result):
    """write overview result to file.

    Args:
        overview_result (pandas.DataFrame): the overview result.
    """
    result_dir = os.path.join(get_root_dir(), "results")
    os.makedirs(result_dir, exist_ok=True)
    result_filename = os.path.join(result_dir, "overview.csv")
    overview_result.to_csv(result_filename, index=None)



import pytest

from fibber.fibber import Fibber


@pytest.mark.slow
def test_fibber():
    arg_dict = {
        "use_gpu_id": 0,
        "gpt2_gpu_id": 0,
        "bert_gpu_id": 0,
        "strategy_gpu_id": 0
    }
    fibber = Fibber(arg_dict, dataset_name="mr", strategy_name="IdentityStrategy",
                    output_dir="exp-pytest")
    text = "test text."
    paraphrases = fibber.paraphrase({"text0": text})[1]
    for item in paraphrases:
        assert item == text



from fibber.resources.download_resources import download_all, download_resources_for_demo
from fibber.resources.resource_utils import (
    get_bert_clf_demo, get_corenlp, get_glove_emb, get_nltk_data, get_stopwords, get_transformers,
    get_universal_sentence_encoder)

__all__ = [
    "get_glove_emb",
    "get_stopwords",
    "get_nltk_data",
    "get_universal_sentence_encoder",
    "get_transformers",
    "get_corenlp",
    "download_all",
    "download_resources_for_demo",
    "get_bert_clf_demo"]



import pytest
import torch

from fibber.metrics.metric_utils import MetricBundle


@pytest.fixture()
def gpu_id():
    if torch.cuda.device_count() > 0:
        return 0
    return -1


@pytest.mark.slow
def test_metric_bundle(gpu_id):
    metric_bundle = MetricBundle(
        use_gpu_id=gpu_id,
        gpt2_gpu_id=gpu_id)
    s1 = "Saturday is the last day in a week"
    s2 = "Sunday is the last day in a week"
    results = metric_bundle.measure_example(s1, s2)
    assert len(results) == 4
